from maubot import Plugin, MessageEvent
from maubot.handlers import command
import re
import aiohttp
import asyncio  # Pro implementaci zpoždění
from datetime import datetime, date, timedelta
import smtplib
from email.mime.text import MIMEText
import unicodedata
import xml.etree.ElementTree as ET
from icalendar import Calendar

class Meshbot(Plugin):
    @command.passive("!l")
    async def reply_to_command_l(self, evt: MessageEvent, val: tuple) -> None:
        command_text = " ".join(val).strip()
        match = re.search(r'!l\s*(.*)', command_text)
        
        if match:
            response = match.group(1).strip()
            if response:
                await evt.respond(response)
            else:
                await evt.respond("Pong")
        else:
            await evt.respond("Pong")

    @command.passive("!o")
    async def reply_to_command_o(self, evt: MessageEvent, val: tuple) -> None:
        command_text = " ".join(val).strip()
        match = re.search(r'!o\s*(\d+)?', command_text)

        if match:
            # Parse the number of minutes (default to 10 if not specified)
            minutes = int(match.group(1)) if match.group(1) else 10

            # Get the current time and format it
            from datetime import datetime
            now = datetime.now().strftime("%H:%M:%S")

            # Confirm the countdown start
            await evt.respond(f"Odpocet {minutes} minut zahájen v {now}.")

            # Wait for the specified number of minutes
            await asyncio.sleep(minutes * 60)

            # Get the current time and format it
            end = datetime.now().strftime("%H:%M:%S")

            # Send the final message
            await evt.respond(f"Dobehl odpocet {minutes} minut ({end}).")
        else:
            await evt.respond("Zadejte príkaz ve formátu !o X, kde X je pocet minut.")


    @command.passive("!d")
    async def reply_to_command_d(self, evt: MessageEvent, val: tuple) -> None:
        current_datetime = datetime.now().strftime("%H:%M:%S %d.%m. %Y")
        await evt.respond(f"{current_datetime}")

    @command.passive("!c")
    async def reply_to_command_c(self, evt: MessageEvent, val: tuple) -> None:
        """
        Příkaz !c pro zobrazení událostí z kalendářů s volitelným parametrem:
        - Bez parametru: dnešní datum.
        - Parametr "zitra": zítřejší datum.
        - Parametr ve formátu "dd.mm.yyyy": konkrétní datum.
        """
        # Kalendáře ke stažení
        calendar_urls = [
            "https://XXXX.XX/CAL.ics",
]
        # Extrahujeme text za c!
        command_text = " ".join(val).strip()
        match = re.search(r'!c\s*(.*)', command_text)
        if not match:
            await evt.respond("Nebyl rozpoznán platný príkaz.")
            return
        param = match.group(1).strip()

        # Zpracování parametru
        if not param:  # Pokud není žádný parametr
            target_date = date.today()
        elif param.lower() == "zitra":
            target_date = date.today() + timedelta(days=1)
        else:
            try:
                target_date = datetime.strptime(param, "%d.%m.%Y").date()
            except ValueError:
                await evt.respond("Neplatný formát data. Použijte dd.mm.yyyy.")
                return


        # Stahování a zpracování kalendářů
        all_events = []
        async with aiohttp.ClientSession() as session:
            for url in calendar_urls:
                try:
                    async with session.get(url) as response:
                        response.raise_for_status()
                        ical_content = await response.read()

                        # Parsování iCal dat
                        cal = Calendar.from_ical(ical_content)
                        for component in cal.walk():
                            if component.name == "VEVENT":
                                event_start = component.get("DTSTART").dt
                                event_end = component.get("DTEND").dt
                                summary = component.get("SUMMARY")

                                # Normalizace dat
                                if isinstance(event_start, datetime):
                                    event_start = event_start.date()
                                if isinstance(event_end, datetime):
                                    event_end = event_end.date()

                                # Filtrování událostí podle data
                                if event_start <= target_date <= event_end:
                                    all_events.append(summary)
                except Exception as e:
                    await evt.respond(f"Chyba nacítání kalendáre z {url}: {e}")
                    return

        # Výstup zkrácený na 200 znaků a převedený do ASCII
        if all_events:
            response = f"{target_date}: " + ", ".join(all_events)
        else:
            response = f"Pro {target_date} nejsou zádné události."

        # Překódování do ASCII a omezení na 200 znaků
        response_ascii = unicodedata.normalize("NFKD", response).encode("ascii", "ignore").decode("ascii")
        response_ascii = response_ascii[:200]  # Oříznutí na 200 znaků

        await evt.respond(response_ascii)
        

    @command.passive("!r")
    async def reply_to_command_r(self, evt: MessageEvent, val: tuple) -> None:
        """Fetch RSS titles from rozhlas.cz/rss and send them without diacritics."""
        rss_url = "https://www.irozhlas.cz/rss/irozhlas"

        async with aiohttp.ClientSession() as session:
            async with session.get(rss_url) as response:
                if response.status != 200:
                    await evt.respond(f"Nepodarilo se nacist RSS z {rss_url}. Status kod: {response.status}")
                    return

                rss_text = await response.text()

        try:
            root = ET.fromstring(rss_text)
            items = root.findall(".//item/title")
            titles = [item.text for item in items if item.text]

            for title in titles:
                title = title.replace('‘', "'").replace('’', "'").replace('“', '"').replace('”', '"')
                # Zkrátit na 200 znaků a odstranit diakritiku přímo zde
                short_title = ''.join(
                    c for c in unicodedata.normalize('NFKD', title[:200])
                    if not unicodedata.combining(c)
                )
                await evt.respond(short_title)
                await asyncio.sleep(5)  # Pauza 5 sekund
        except Exception as e:
            await evt.respond(f"Chyba pri zpracovani RSS: {str(e)}")


    @command.passive("!p")
    async def reply_to_command_p(self, evt: MessageEvent, val: tuple) -> None:
        url = "https://postolka.klikva.net/panel"
        
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                if response.status == 200:
                    text = await response.text()
                    await evt.respond(f"{text}")
                else:
                    await evt.respond(f"Nefunguje https://postolka.klikva.net/panel. Status kod: {response.status}")

    @command.passive("!a")
    async def reply_to_command_a(self, evt: MessageEvent, val: tuple) -> None:
        command_text = " ".join(val).strip()
        match = re.search(r'!a\s*(.*)', command_text)

        if match:
            query = match.group(1).strip()
            if query:
                api_key = "XXXXXXXXXXXXXXXXXXX"  # Zde vložte váš OpenAI API klíč
                url = "https://api.openai.com/v1/chat/completions"
                headers = {
                    "Content-Type": "application/json",
                    "Authorization": f"Bearer {api_key}"
                }
                data = {
                    "model": "gpt-4o-2024-08-06",
                    "messages": [
                        {"role": "system", "content": "Jsi exaktní vysokoškolský vědec, odpovídáš bez diakritiky a to pouze ve 200 znacích."},
                        {"role": "user", "content": query}
                    ],
                    "temperature": 0.0
                }

                async with aiohttp.ClientSession() as session:
                    async with session.post(url, headers=headers, json=data) as response:
                        if response.status == 200:
                            result = await response.json()
                            answer = result['choices'][0]['message']['content']
                            normalized_answer = unicodedata.normalize('NFKD', answer).encode('ASCII', 'ignore').decode('ASCII')
                            truncated_answer = normalized_answer[:200]
                            await evt.respond(truncated_answer)
                        else:
                            await evt.respond(f"Chyba pri komunikaci s API. Status kod: {response.status}")
            else:
                await evt.respond("Chybi dotaz za prikazem !a.")
        else:
            await evt.respond("Chybi dotaz za prikazem !a.")

    @command.passive("!t")
    async def reply_to_command_t(self, evt: MessageEvent, val: tuple) -> None:
        """
        Odpovídá na příkaz !t a odesílá e-mail na pevně zadanou adresu.
        Text za !t se odesílá jako tělo zprávy s pevně stanoveným předmětem a přidáním aktuálního data.
        """
        # Extrahujeme text za !t
        command_text = " ".join(val).strip()
        match = re.search(r'!t\s+(.*)', command_text)

        if match:
            body_text = match.group(1)

            # SMTP nastavení
            smtp_server = "smtp.klikva.cz"
            smtp_port = 465  # Výchozí port pro SSL
            smtp_user = "XXXXXXXXXXX"
            smtp_password = "iXXXXXXXXXXXXX"

            # Nastavení pevně zadané e-mailové adresy a předmětu
            recipient = "XXXXXXXXXX"
            subject = "iXXXXXXXXX"

            # Vytvoření těla zprávy
            current_date = datetime.now().strftime("%d.%m.%Y")
            body = f"{current_date};{body_text}"

            # Vytvoříme zprávu
            msg = MIMEText(body)
            msg['From'] = smtp_user
            msg['To'] = recipient
            msg['Subject'] = subject

            # Pokus o odeslání e-mailu
            try:
                with smtplib.SMTP_SSL(smtp_server, smtp_port) as server:
                    server.login(smtp_user, smtp_password)
                    server.sendmail(smtp_user, recipient, msg.as_string())
                    await evt.respond(f"Ukol na XXXX.XXX odeslán.")
                    
                    # Čekání 2 sekundy
                    await asyncio.sleep(1)

                    # První volání URL
                    async with aiohttp.ClientSession() as session:
                        async with session.get("http://XXXX.XXX/iXXXX.php") as response1:
                            if response1.status != 200:
                                await evt.respond(f"Synchronizace s IMAP selhala. HTTP status: {response1.status}")

                        # Druhé volání URL
                        async with session.get("https://XXXX.XXX/XXX.php") as response2:
                            if response2.status != 200:
                                await evt.respond(f"CRON aktualizace selhala. HTTP status: {response2.status}")

            except Exception as e:
                await evt.respond(f"Chyba při odesílání e-mailu nebo volání URL: {e}")
        else:
            await evt.respond("Neplatný formát příkazu! Použijte: !t <text>")


    @command.passive("!k")
    async def reply_to_command_k(self, evt: MessageEvent, val: tuple) -> None:
        """
        Odpovídá na příkaz !k a odesílá e-mail na pevně zadanou adresu.
        Text za !k se odesílá jako tělo zprávy s pevně stanoveným předmětem.
        """
        # Extrahujeme text za !t
        command_text = " ".join(val).strip()
        match = re.search(r'!k\s+(.*)', command_text)

        if match:
            body_text = match.group(1)

            # SMTP nastavení
            smtp_server = "smtp.klikva.cz"
            smtp_port = 465  # Výchozí port pro SSL
            smtp_user = "XXXXXXXXXXXXXXX"
            smtp_password = "XXXXXXXXXXXX"

            # Nastavení pevně zadané e-mailové adresy a předmětu
            recipient = "XXXXXXXXXXXXXXX"
            subject = "XXXXXXXXX"

            # Vytvoření těla zprávy
            current_date = datetime.now().strftime("%d.%m.%Y")
            body = f"{body_text}"

            # Vytvoříme zprávu
            msg = MIMEText(body)
            msg['From'] = smtp_user
            msg['To'] = recipient
            msg['Subject'] = subject

            # Pokus o odeslání e-mailu
            try:
                with smtplib.SMTP_SSL(smtp_server, smtp_port) as server:
                    server.login(smtp_user, smtp_password)
                    server.sendmail(smtp_user, recipient, msg.as_string())
                    await evt.respond(f"Schuzka do kalendare na XXXX.XXX odeslána.")
                    
                    # Čekání 2 sekundy
                    await asyncio.sleep(1)

                    # První volání URL
                    async with aiohttp.ClientSession() as session:
                        async with session.get("http://XXXX.XXX/XXXX.php") as response1:
                            if response1.status != 200:
                                await evt.respond(f"Synchronizace s IMAP selhala. HTTP status: {response1.status}")

                        # Druhé volání URL
                        async with session.get("https://XXXX.XXX/XXXX.php") as response2:
                            if response2.status != 200:
                                await evt.respond(f"CRON aktualizace selhala. HTTP status: {response2.status}")

            except Exception as e:
                await evt.respond(f"Chyba při odesílání e-mailu nebo volání URL: {e}")
        else:
            await evt.respond("Neplatný formát příkazu! Použijte: !t <text>")


    @command.passive("!z")
    async def reply_to_command_z(self, evt: MessageEvent, val: tuple) -> None:
        """
        Odpovídá na příkaz !z a odesílá e-mail na pevně zadanou adresu.
        Text za !z se odesílá jako tělo zprávy s pevně stanoveným předmětem a přidáním aktuálního data.
        """
        # Extrahujeme text za !b
        command_text = " ".join(val).strip()
        match = re.search(r'!z\s+(.*)', command_text)

        if match:
            body_text = match.group(1)

            # SMTP nastavení
            smtp_server = "smtp.klikva.cz"
            smtp_port = 465  # Výchozí port pro SSL
            smtp_user = "XXXXXXXXXXXXXXX"
            smtp_password = "XXXXXXX"

            # Nastavení pevně zadané e-mailové adresy a předmětu
            recipient = "XXXXXXXXXXXXXXX"
            subject = "head:zapisky"

            # Vytvoření těla zprávy
            current_date = datetime.now().strftime("%d.%m.%Y")
            body = f" * '''{current_date}''' -- {body_text}"

            # Vytvoříme zprávu
            msg = MIMEText(body)
            msg['From'] = smtp_user
            msg['To'] = recipient
            msg['Subject'] = subject

            # Pokus o odeslání e-mailu
            try:
                with smtplib.SMTP_SSL(smtp_server, smtp_port) as server:
                    server.login(smtp_user, smtp_password)
                    server.sendmail(smtp_user, recipient, msg.as_string())
                    await evt.respond(f"Zapisky na XXXX.XXX odeslány.")
                    
                    # Čekání 2 sekundy
                    await asyncio.sleep(1)

                    # První volání URL
                    async with aiohttp.ClientSession() as session:
                        async with session.get("http://XXXX.XXX/XXXX.php") as response1:
                            if response1.status != 200:
                                await evt.respond(f"Synchronizace s IMAP selhala. HTTP status: {response1.status}")

                        # Druhé volání URL
                        async with session.get("https://XXXX.XXX/XXX.php") as response2:
                            if response2.status != 200:
                                await evt.respond(f"CRON aktualizace selhala. HTTP status: {response2.status}")

            except Exception as e:
                await evt.respond(f"Chyba při odesílání e-mailu nebo volání URL: {e}")
        else:
            await evt.respond("Neplatný formát příkazu! Použijte: !z <text>")

 
    @command.passive("!b")
    async def reply_to_command_b(self, evt: MessageEvent, val: tuple) -> None:
        """
        Odpovídá na příkaz !b a odesílá e-mail na pevně zadanou adresu.
        Text za !b se odesílá jako tělo zprávy s pevně stanoveným předmětem a přidáním aktuálního data.
        """
        # Extrahujeme text za !b
        command_text = " ".join(val).strip()
        match = re.search(r'!b\s+(.*)', command_text)

        if match:
            body_text = match.group(1)

            # SMTP nastavení
            smtp_server = "smtp.klikva.cz"
            smtp_port = 465  # Výchozí port pro SSL
            smtp_user = "iXXXXX"
            smtp_password = "XXXXXXXXXX"

            # Nastavení pevně zadané e-mailové adresy a předmětu
            recipient = "XXXXXXXXXX"
            subject = "XXXXXXXXXXXXX"

            # Vytvoření těla zprávy
            current_date = datetime.now().strftime("%d.%m.%Y")
            body = f" * '''{current_date}''' -- {body_text}"

            # Vytvoříme zprávu
            msg = MIMEText(body)
            msg['From'] = smtp_user
            msg['To'] = recipient
            msg['Subject'] = subject

            # Pokus o odeslání e-mailu
            try:
                with smtplib.SMTP_SSL(smtp_server, smtp_port) as server:
                    server.login(smtp_user, smtp_password)
                    server.sendmail(smtp_user, recipient, msg.as_string())
                    await evt.respond(f"Aktualita na XXXXXXXXXXXXXXX odeslána.")
                    
                    # Čekání 2 sekundy
                    await asyncio.sleep(1)

                    # První volání URL
                    async with aiohttp.ClientSession() as session:
                        async with session.get("http://XXXXXXXXXXXXXXX/XXX.php") as response1:
                            if response1.status != 200:
                                await evt.respond(f"Synchronizace s IMAP selhala. HTTP status: {response1.status}")

                        # Druhé volání URL
                        async with session.get("https://XXXXXXXXXXXXXXX/XXXX.php") as response2:
                            if response2.status != 200:
                                await evt.respond(f"CRON aktualizace selhala. HTTP status: {response2.status}")

            except Exception as e:
                await evt.respond(f"Chyba při odesílání e-mailu nebo volání URL: {e}")
        else:
            await evt.respond("Neplatný formát příkazu! Použijte: !b <text>")

    @command.passive("!m")
    async def reply_to_command_m(self, evt: MessageEvent, val: tuple) -> None:
        """
        Odpovídá na příkaz !m a odesílá e-mail na zadanou adresu.
        První slovo za !m je e-mailová adresa, druhé subject a zbytek je tělo zprávy.
        """
        # Extrahujeme detaily e-mailu
        command_text = " ".join(val).strip()
        match = re.search(r'!m\s+(\S+)\s+(\S+)\s+(.*)', command_text)

        if match:
            recipient = match.group(1)
            subject = match.group(2)
            body = match.group(3)
            
            # SMTP nastavení - získáme z konfigurace
            smtp_server = "smtp.klikva.cz"
            smtp_port = "465"  # Výchozí port pro TLS
            smtp_user = "XXXXXXXXXXXXXXX"
            smtp_password = "XXXXX"

            # Vytvoříme zprávu
            msg = MIMEText(body)
            msg['From'] = smtp_user
            msg['To'] = recipient
            msg['Subject'] = subject

            # Pokus o odeslání e-mailu
            try:
                with smtplib.SMTP_SSL(smtp_server, smtp_port) as server:
                    server.login(smtp_user, smtp_password)
                    server.sendmail(smtp_user, recipient, msg.as_string())
                    await evt.respond(f"E-mail na {recipient} uspesne odeslan.")
            except Exception as e:
                await evt.respond(f"Chyba pri odesilani e-mailu: {e}")
        else:
            await evt.respond("Neplatny format príkazu! Použijte: !m <email> <subject> <body>")

    @command.passive("!s")
    async def reply_to_command_s(self, evt: MessageEvent, val: tuple) -> None:
        """
        Odpovídá na příkaz !s a odesílá SMS zprávu na zadané telefonní číslo.
        První slovo za !s je telefonní číslo, zbytek je text zprávy.
        """
        command_text = " ".join(val).strip()
        match = re.search(r'!s\s+(\d+)\s+(.*)', command_text)

        if match:
            phone_number = match.group(1)
            message_body = match.group(2).strip()

            # Kontrola platnosti telefonního čísla
            if len(phone_number) < 9 or not phone_number.isdigit():
                await evt.respond(f"Chyba: Neplatne telefonni cislo {phone_number}.")
                return
            
            # Twilio nastavení - získáme z konfigurace
            twilio_sid = "XXXXXXXXXX"
            twilio_token = "iXXXXXXXXXXXXXX"
            messaging_service_sid = "XXXXXXXXXXXXXXXXXX"
            url = f"https://api.twilio.com/2010-04-01/Accounts/{twilio_sid}/Messages.json"

            # Tělo požadavku pro Twilio API
            data = {
                "To": f"+420{phone_number}",
                "MessagingServiceSid": messaging_service_sid,
                "Body": message_body[:500]  # Maximální délka zprávy 500 znaků
            }

            # Odeslání SMS přes Twilio API
            async with aiohttp.ClientSession() as session:
                async with session.post(url, data=data, auth=aiohttp.BasicAuth(twilio_sid, twilio_token)) as response:
                    if response.status == 201:
                        await evt.respond(f"SMS na cislo {phone_number} odeslana.")
                    else:
                        await evt.respond(f"Chyba pri odesilani SMS. Status kod: {response.status}")
        else:
            await evt.respond("Spatny format prikazu! Pouzijte: !s <telefonni cislo> <zprava>")
